"""Optional values.

Type Option represents an optional value: every Option is either
:meth:`Option.Some` and contains a value, or :meth:`Option.None_`, and does not.
Option types are very common in Rust code, as they have a number of uses:

- Initial values
- Return values for functions that are not defined over their entire input range
  (partial functions)
- Return value for otherwise reporting simple errors, where ``None_`` is
  returned on error
- Optional struct fields
- Struct fields that can be loaned or "taken"
- Optional function arguments
- Nullable pointers
- Swapping things out of difficult situations

Options are commonly paired with pattern matching to query the presence of a
value and take action, always accounting for the ``None_`` case.

>>> def divide(numerator: float, denominator: float) -> Option[float]:
...     if denominator == 0.0:
...         return None_()
...     return Some(numerator / denominator)
...
>>> result = divide(2.0, 3.0)
>>>
>>> if result.is_some():
...     print(f"Result: {result.unwrap()}")
... else:
...     print(f"Cannot divide by 0")
...
Result: 0.6666666666666666

Examples
--------

Basic pattern matching on :class:`Option`:

>>> msg = Some("howdy")
>>>
>>> if msg.is_some():
...     print(msg.unwrap())
...
howdy
>>> unwrapped_msg = msg.unwrap_or("default message")
>>> unwrapped_msg
'howdy'

Initialize a result to ``None_`` before a loop:

>>> from dataclasses import dataclass
>>>
>>> @dataclass
... class Kingdom:
...     size: int
...     name: str
...
>>> class PlantKingdom(Kingdom): pass
...
>>> class AnimalKingdom(Kingdom): pass
...
>>> all_the_big_things = [
...     PlantKingdom(250, "redwood"),
...     PlantKingdom(230, "noble fir"),
...     PlantKingdom(229, "sugar pine"),
...     AnimalKingdom(25, "blue whale"),
...     AnimalKingdom(19, "fin whale"),
...     AnimalKingdom(15, "north pacific right whale"),
... ]
>>>
>>> # We're going to search for the name of the biggest animal, but to start
>>> # with we've just got `None_`
>>> name_of_biggest_animal: Option[str] = None_()
>>> size_of_biggest_animal = 0
>>> for big_thing in all_the_big_things:
...     if isinstance(big_thing, AnimalKingdom) \\
...             and big_thing.size > size_of_biggest_animal:
...         size_of_biggest_animal = big_thing.size
...         name_of_biggest_animal = Some(big_thing.name)
...
>>> if name_of_biggest_animal.is_some():
...     print(f"the biggest animal is {name_of_biggest_animal.unwrap()}")
... else:
...     print("there are no animals :(")
...
the biggest animal is blue whale
"""

from typing import (
    Callable,
    Generic,
    Iterator,
    Optional,
    Tuple,
    TypeVar,
    Union,
)

from .ref import Ref
from ._utils import nothing, Nothing

__all__ = ("Option", "Some", "None_")

T = TypeVar("T")
U = TypeVar("U")
E = TypeVar("E")


class Option(Generic[T]):
    """The ``Option`` type. See the :mod:`module-level documentation
    <result.option>` for more."""

    __slots__ = ("_value",)
    _value: Union[Nothing, T]

    def __init__(self, value: Union[Nothing, T] = nothing):
        """This constructor should not be called directly.

        Instead, use :meth:`Some` or :meth:`None_`
        """
        self._value = value

    def is_some(self) -> bool:
        """Returns :obj:`True` if the option is a ``Some`` value.

        >>> x: Option[int] = Some(2)
        >>> x.is_some()
        True
        >>> x: Option[int] = None_()
        >>> x.is_some()
        False
        """
        return self._value is not nothing

    def is_none(self) -> bool:
        """Returns :obj:`True` if the option is a ``None_`` value.

        >>> x: Option[int] = Some(2)
        >>> x.is_none()
        False
        >>> x: Option[int] = None_()
        >>> x.is_none()
        True
        """
        return self._value is nothing

    def expect(self, msg: str) -> T:
        """Returns the contained ``Some`` value.

        :raises AssertionError: Raised if the value is a ``None_``, with a
            custom message provided by ``msg``.

        >>> x = Some("value")
        >>> x.expect("fruits are healthy")
        'value'
        >>> x: Option[str] = None_()
        >>> x.expect("fruits are healthy")
        Traceback (most recent call last):
            ...
        AssertionError: fruits are healthy
        """
        if self._value is nothing:
            raise AssertionError(msg)
        return self._value

    def unwrap(self) -> T:
        """Returns the contained ``Some`` value.

        Because this function may panic, its use is generally discouraged.
        Instead, prefer to use pattern matching and handle the ``None_`` case
        explicitly, or call :meth:`unwrap_or` or :meth:`unwrap_or_else`.

        :raises AssertionError: Raised if the self value is ``None_``.

        >>> x = Some("air")
        >>> x.unwrap()
        'air'
        >>> x: Option[str] = None_()
        >>> x.unwrap()
        Traceback (most recent call last):
            ...
        AssertionError: called `Option.unwrap` on a `None_` value
        """
        if self._value is nothing:
            raise AssertionError("called `Option.unwrap` on a `None_` value")
        return self._value

    def unwrap_or(self, default: T) -> T:
        """Returns the contained ``Some`` value or a provided default.

        Arguments passed to ``unwrap_or`` are eagerly evaluated; if you are
        passing the result of a function call, it is recommended to use
        :meth:`unwrap_or_else`, which is lazily evaluated.

        >>> Some("car").unwrap_or("bike")
        'car'
        >>> Option[str].None_().unwrap_or("bike")
        'bike'
        """
        if self._value is nothing:
            return default
        return self._value

    def unwrap_or_else(self, f: Callable[[], T]) -> T:
        """Returns the contained ``Some`` value or computes it from a closure.

        >>> k = 10
        >>> Some(4).unwrap_or_else(lambda: 2 * k)
        4
        >>> Option[int].None_().unwrap_or_else(lambda: 2 * k)
        20
        """
        if self._value is nothing:
            return f()
        return self._value

    def map(self, f: Callable[[T], U]) -> "Option[U]":
        """Maps an ``Option[T]`` to ``Option[U]`` by applying a function to a
        contained value.

        Converting an ``Option[str]`` into an ``Option[int]``:

        >>> maybe_some_string = Some("Hello, World!")
        >>> maybe_some_len = maybe_some_string.map(len)
        >>> maybe_some_len
        Some(13)
        """
        if self._value is nothing:
            return None_()
        return Some(f(self._value))

    def map_or(self, default: U, f: Callable[[T], U]) -> U:
        """Applies a function to the contained value (if any), or returns the
        provided default (if not).

        Arguments passed to ``map_or`` are eagerly evaluated; if you are passing
        the result of a function call, it is recommended to use
        :meth:`map_or_else`, which is lazily evaluated.

        >>> x = Some("foo")
        >>> x.map_or(42, len)
        3
        >>> x: Option[str] = None_()
        >>> x.map_or(42, len)
        42
        """
        if self._value is nothing:
            return default
        return f(self._value)

    def map_or_else(self, default: Callable[[], U], f: Callable[[T], U]) -> U:
        """Applies a function to the contained value (if any), or computes a
        default (if not).

        >>> k = 21
        >>>
        >>> x = Some("foo")
        >>> x.map_or_else(lambda: 2 * k, len)
        3
        >>> x: Option[str] = None_()
        >>> x.map_or_else(lambda: 2 * k, len)
        42
        """
        if self._value is nothing:
            return default()
        return f(self._value)

    def ok_or(self, err: E) -> "Result[T, E]":
        """Transforms the ``Option[T]`` into a :class:`Result[T, E]
        <result.result.Result>`, mapping ``Some(v)`` to ``Ok(v)`` and ``None_``
        to ``Err(err)``.

        Arguments passed to ``ok_or`` are eagerly evaluated; if you are passing
        the result of a function call, it is recommended to use
        :meth:`ok_or_else`, which is lazily evaluated.

        >>> x = Some("foo")
        >>> x.ok_or(0)
        Ok('foo')
        >>> x: Option[str] = None_()
        >>> x.ok_or(0)
        Err(0)
        """
        if self._value is nothing:
            return Result.Err(err)
        return Result.Ok(self._value)

    def ok_or_else(self, err: Callable[[], E]) -> "Result[T, E]":
        """Transforms the ``Option[T]`` into a :class:`Result[T, E]
        <result.result.Result>`, mapping ``Some(v)`` to ``Ok(v)`` and ``None_``
        to ``Err(err())``.

        >>> x = Some("foo")
        >>> x.ok_or_else(lambda: 0)
        Ok('foo')
        >>> x: Option[str] = None_()
        >>> x.ok_or_else(lambda: 0)
        Err(0)
        """
        if self._value is nothing:
            return Result.Err(err())
        return Result.Ok(self._value)

    def iter(self) -> Iterator[T]:
        """Returns an iterator over the possibly contained value.

        >>> x = Some(4)
        >>> next(x.iter())
        4
        >>> x: Option[int] = None_()
        >>> next(x.iter())
        Traceback (most recent call last):
            ...
        StopIteration
        """
        if self._value is not nothing:
            yield self._value

    def __iter__(self):
        """Returns an iterator over the possibly contained value.

        >>> x = Some(4)
        >>> next(iter(x))
        4
        >>> x: Option[int] = None_()
        >>> next(iter(x))
        Traceback (most recent call last):
            ...
        StopIteration
        """
        return self.iter()

    def and_(self, optb: "Option[U]") -> "Option[U]":
        """Returns ``None_`` if the option is ``None_``, otherwise returns
        ``optb``.

        >>> x = Some(2)
        >>> y: Option[str] = None_()
        >>> x.and_(y)
        None_
        >>> x: Option[int] = None_()
        >>> y = Some("foo")
        >>> x.and_(y)
        None_
        >>> x = Some(2)
        >>> y = Some("foo")
        >>> x.and_(y)
        Some('foo')
        >>> x: Option[int] = None_()
        >>> y: Option[str] = None_()
        >>> x.and_(y)
        None_
        """
        if self._value is nothing:
            return None_()
        return optb

    def and_then(self, f: Callable[[T], "Option[U]"]) -> "Option[U]":
        """Returns ``None_`` if the option is ``None_``, otherwise calls ``f``
        with the wrapped value and returns the result.

        Some languages call this operation flatmap.

        >>> def sq(x: int) -> Option[int]: return Some(x * x)
        ...
        >>> def nope(_: int) -> Option[int]: return None_()
        ...
        >>> Some(2).and_then(sq).and_then(sq)
        Some(16)
        >>> Some(2).and_then(sq).and_then(nope)
        None_
        >>> Some(2).and_then(nope).and_then(sq)
        None_
        >>> None_().and_then(sq).and_then(sq)
        None_
        """
        if self._value is nothing:
            return None_()
        return f(self._value)

    def filter(self, predicate: Callable[[T], bool]) -> "Option[T]":
        """Returns ``None_`` if the option is ``None_``, otherwise calls
        ``predicate`` with the wrapped value and returns:

        - ``Some(t)`` if ``predicate`` returns ``True`` (where ``t`` is the
          wrapped value, and
        - ``None_`` if ``predicate`` returns ``False``.

        This function works similar to :py:func:`python:filter`. You can
        imagine the ``Option[T]`` being an iterator over one or zero elements.
        ``filter()`` lets you decide which elements to keep.

        >>> def is_even(n: int) -> bool: return n % 2 == 0
        ...
        >>> None_().filter(is_even)
        None_
        >>> Some(3).filter(is_even)
        None_
        >>> Some(4).filter(is_even)
        Some(4)
        """
        if self._value is nothing or not predicate(self._value):
            return None_()
        return Some(self._value)

    def or_(self, optb: "Option[T]") -> "Option[T]":
        """Returns the option if it contains a value, otherwise returns
        ``optb``.

        Arguments passed to ``or`` are eagerly evaluated; if you are passing the
        result of a function call, it is recommended to use :meth:`or_else`,
        which is lazily evaluated.

        >>> x = Some(2)
        >>> y: Option[int] = None_()
        >>> x.or_(y)
        Some(2)
        >>> x: Option[int] = None_()
        >>> y = Some(100)
        >>> x.or_(y)
        Some(100)
        >>> x = Some(2)
        >>> y = Some(100)
        >>> x.or_(y)
        Some(2)
        >>> x: Option[int] = None_()
        >>> y: Option[int] = None_()
        >>> x.or_(y)
        None_
        """
        if self._value is nothing:
            return optb
        return Some(self._value)

    def or_else(self, f: Callable[[], "Option[T]"]) -> "Option[T]":
        """Returns the option if it contains a value, otherwise calls ``f`` and
        returns the result.

        >>> def nobody() -> Option[str]: return None_()
        ...
        >>> def vikings() -> Option[str]: return Some("vikings")
        ...
        >>> Some("barbarians").or_else(vikings)
        Some('barbarians')
        >>> Option[str].None_().or_else(vikings)
        Some('vikings')
        >>> Option[str].None_().or_else(nobody)
        None_
        """
        if self._value is nothing:
            return f()
        return Some(self._value)

    def xor(self, optb: "Option[T]") -> "Option[T]":
        """Returns ``Some`` if exactly one of ``self``, ``optb`` is ``Some``,
        otherwise returns ``None_``.

        >>> x = Some(2)
        >>> y: Option[int] = None_()
        >>> x.xor(y)
        Some(2)
        >>> x: Option[int] = None_()
        >>> y = Some(2)
        >>> x.xor(y)
        Some(2)
        >>> x = Some(2)
        >>> y = Some(2)
        >>> x.xor(y)
        None_
        >>> x: Option[int] = None_()
        >>> y: Option[int] = None_()
        >>> x.xor(y)
        None_
        """
        if self._value is nothing and optb._value is not nothing:
            return Some(optb._value)
        if self._value is not nothing and optb._value is nothing:
            return Some(self._value)
        return None_()

    def _make_value_ref(self) -> Ref[T]:
        def get_value() -> T:
            if self._value is nothing:
                raise AssertionError("Reading reference to None_ value")
            return self._value

        def set_value(value: T):
            if self._value is nothing:
                raise AssertionError("Writing to reference to None_ value")
            self._value = value

        return Ref(get_value, set_value)

    def get_or_insert(self, v: T) -> Ref[T]:
        """Inserts ``v`` into the option if it is ``None_``, then returns
        a reference to the contained value.

        >>> x: Option[int] = None_()
        >>> y = x.get_or_insert(5)
        >>> y.get()
        5
        >>> y.set(7)
        >>> x
        Some(7)
        """
        if self._value is nothing:
            self._value = v
        return self._make_value_ref()

    def get_or_insert_with(self, f: Callable[[], T]) -> Ref[T]:
        """Inserts a value computed from ``f`` into the option if it is
        ``None_``, then returns a reference to the contained value.

        >>> x: Option[int] = None_()
        >>> y = x.get_or_insert_with(lambda: 5)
        >>> y.get()
        5
        >>> y.set(7)
        >>> x
        Some(7)
        """
        if self._value is nothing:
            self._value = f()
        return self._make_value_ref()

    def take(self) -> "Option[T]":
        """Takes the value out of the option, leaving a ``None_`` in its place.

        >>> x = Some(2)
        >>> y = x.take()
        >>> x
        None_
        >>> y
        Some(2)
        >>> x: Option[int] = None_()
        >>> y = x.take()
        >>> x
        None_
        >>> y
        None_
        """
        if self._value is nothing:
            return None_()
        value, self._value = self._value, nothing
        return Some(value)

    def replace(self, value: T) -> "Option[T]":
        """Replaces the actual value in the option by the value given in
        parameter, returning the old value if present, leaving a ``Some`` in its
        place without deinitializing either one.

        >>> x = Some(2)
        >>> old = x.replace(5)
        >>> x
        Some(5)
        >>> old
        Some(2)
        >>> x = Option[int].None_()
        >>> old = x.replace(3)
        >>> x
        Some(3)
        >>> old
        None_
        """
        old_value, self._value = self._value, value
        if old_value is nothing:
            return None_()
        return Some(old_value)

    def zip(self, other: "Option[U]") -> "Option[Tuple[T, U]]":
        """Zips ``self`` with another :class:`Option`.

        If ``self`` is ``Some(s)`` and ``other`` is ``Some(o)``, this method
        returns ``Some((s, o))``. Otherwise, ``None_`` is returned.

        >>> x = Some(1)
        >>> y = Some("hi")
        >>> z = Option[int].None_()
        >>> x.zip(y)
        Some((1, 'hi'))
        >>> x.zip(z)
        None_
        """
        if self._value is nothing or other._value is nothing:
            return None_()
        return Some((self._value, other._value))

    def __repr__(self):
        if self._value is nothing:
            return "None_"
        return f"Some({repr(self._value)})"

    @classmethod
    def Some(cls, value: T) -> "Option[T]":
        """Some value ``T``"""
        return cls(value)

    @classmethod
    def None_(cls) -> "Option[T]":
        """No value"""
        return cls()

    @classmethod
    def from_optional(cls, opt: Optional[T]) -> "Option[T]":
        """Get an ``Option[T]`` from an :class:`Optional[T] <Optional>`.

        If ``opt`` is :obj:`None`, return :meth:`None_`, otherwise return
        :meth:`Some(obj) <Some>`. This does not come from Rust's API.

        >>> from typing import Optional
        >>> x: Optional[int] = None
        >>> Option.from_optional(x)
        None_
        >>> x: Optional[int] = 42
        >>> Option.from_optional(x)
        Some(42)
        """
        if opt is None:
            return cls.None_()
        return cls.Some(opt)


Some = Option.Some
None_ = Option.None_


from .result import Result  # noqa E402



import enum
import operator
from collections import Hashable
from typing import cast, Any, Callable, TypeVar, Union
from typing_extensions import final, Final

T = TypeVar("T")


def dependent_hash(attr_name: Union[Callable[[T], str], str]) -> Callable:
    def make_hash(self: T):
        value = getattr(self, attr_name(self) if callable(attr_name) else attr_name)
        if not isinstance(value, Hashable):
            return None

        def __hash__():
            return hash((type(self), value))

        return __hash__

    return cast(Callable, property(make_hash))


def dependent_ord(attr_name: Union[Callable[[T], str], str]):
    def make_ordering_function(name: str) -> Callable[[T, Any], bool]:
        assert hasattr(operator, name)
        op = getattr(operator, name)

        def ordering_function(self: T, other: Any) -> bool:
            if type(other) is not type(self):
                return NotImplemented
            attr = attr_name(self) if callable(attr_name) else attr_name
            a, b = getattr(self, attr), getattr(other, attr)
            return op(a, b)

        ordering_function.__name__ = name
        return ordering_function

    def decorate(cls: T) -> T:
        ordering_attrs = ("__lt__", "__le__", "__eq__", "__ne__", "__ge__", "__gt__")
        for attr in ordering_attrs:
            setattr(cls, attr, make_ordering_function(attr))
        return cls

    return decorate


@final
class Nothing(enum.Enum):
    nothing = 0


nothing: Final = Nothing.nothing



from typing import Callable, Generic, TypeVar

T = TypeVar("T")


class Ref(Generic[T]):
    """A simulation of a pointer using closures"""

    __slots__ = ("_get", "_set")

    def __init__(self, get: Callable[[], T], set: Callable[[T], None]):
        self._get = get
        self._set = set

    def get(self) -> T:
        """Deference this ref to get the current value at the remote location."""
        return self._get()

    def set(self, value: T):
        """Update the value at the remote location."""
        self._set(value)



"""Error handling with the Result type.

:class:`Result[T, E] <Result>` is the type used for returning and propagating
errors. It is an enum with the variants, :meth:`Ok(T) <Ok>`, representing
success and containing a value, and :meth:`Err(E) <Err>`, representing error and
containing an error value.

Examples
--------

A simple function returning :class:`Result` might be defined and used like so:

>>> from enum import auto, Enum
>>>
>>> class Version(Enum):
...     version_1 = auto()
...     version_2 = auto()
...
>>> def parse_version(header: bytes) -> Result[Version, str]:
...     if len(header) == 0:
...         return Err("invalid header length")
...     elif header[0] == 1:
...         return Ok(Version.version_1)
...     elif header[0] == 2:
...         return Ok(Version.version_2)
...     else:
...         return Err("invalid version")
...
>>> version = parse_version(bytes([1, 2, 3, 4]))
>>> if version.is_ok():
...     print(f"working with version: {version.unwrap()}")
... else:
...     print(f"error parsing header: {version.unwrap_err()}")
...
working with version: Version.version_1

Pattern matching on :class:`Result`\\s is clear and straightforward for simple
cases (in Rust), but ``Result`` comes with some convenience methods that make
working with it more succinct.

>>> good_result: Result[int, int] = Ok(10)
>>> bad_result: Result[int, int] = Err(10)
>>>
>>> # The `is_ok` and `is_err` methods do what they say.
>>> good_result.is_ok() and not good_result.is_err()
True
>>> bad_result.is_err() and not bad_result.is_ok()
True
>>>
>>> # `map` consumes the `Result` and produces another.
>>> good_result: Result[int, int] = good_result.map(lambda i: i + 1)
>>> bad_result: Result[int, int] = bad_result.map(lambda i: i - 1)
>>>
>>> # Use `and_then` to continue their computation.
>>> good_result: Result[bool, int] = good_result.and_then(lambda i: Ok(i == 11))
>>>
>>> # Use `or_else` to handle the error.
>>> bad_result: Result[int, int] = bad_result.or_else(lambda i: Ok(i + 20))
>>>
>>> # Consume the result and return the contents with `unwrap`.
>>> final_awesome_result = good_result.unwrap()
>>> final_awesome_result
True
"""

import enum
from typing import Callable, Generic, Iterator, TypeVar, Union
from typing_extensions import final

from .option import Option
from ._utils import dependent_hash, dependent_ord, nothing, Nothing

__all__ = ("Result", "Ok", "Err")

T = TypeVar("T")
E = TypeVar("E")
U = TypeVar("U")
F = TypeVar("F")


@final
class _ResultState(enum.Enum):
    ok = enum.auto()
    err = enum.auto()


def _result_value_attr(r: "Result[T, E]") -> str:
    return "_left" if r._state is _ResultState.err else "_right"


@dependent_ord(_result_value_attr)
class Result(Generic[T, E]):
    """``Result`` is a type that represents either success (:meth:`Ok`) or
    failure (:meth:`Err`).
    """

    __slots__ = ("_state", "_left", "_right")
    _state: _ResultState
    _left: E
    _right: T

    def __init__(
        self,
        *,
        left: Union[Nothing, E] = nothing,
        right: Union[Nothing, T] = nothing,
    ):
        """This constructor should not be called directly.

        Instead, use :meth:`Ok` or :meth:`Err`.
        """
        if left is nothing and right is not nothing:
            self._state = _ResultState.ok
            self._right = right
        elif left is not nothing and right is nothing:
            self._state = _ResultState.err
            self._left = left
        else:
            raise AssertionError(
                "One and only one of `left` and `right` must be supplied to "
                "Result.__init__"
            )

    def is_ok(self) -> bool:
        """Returns :obj:`True` if the result is :meth:`Ok`.

        Basic usage:

        >>> x: Result[int, str] = Ok(-3)
        >>> x.is_ok()
        True
        >>>
        >>> x: Result[int, str] = Err("Some error message")
        >>> x.is_ok()
        False
        """
        return self._state is _ResultState.ok

    def is_err(self) -> bool:
        """Returns :obj:`True` if the result is :meth:`Err`.

        Basic usage:

        >>> x: Result[int, str] = Ok(-3)
        >>> x.is_err()
        False
        >>>
        >>> x: Result[int, str] = Err("Some error message")
        >>> x.is_err()
        True
        """
        return self._state is _ResultState.err

    def ok(self) -> Option[T]:
        """Converts from ``Result[T, E]`` to :class:`Option[T]
        <result.option.Option>`.

        Converts ``self`` into ``Option[T]``, discarding the error, if any.

        Basic usage:

        >>> x: Result[int, str] = Ok(2)
        >>> x.ok()
        Some(2)
        >>>
        >>> x: Result[int, str] = Err("Nothing here")
        >>> x.ok()
        None_
        """
        if self._state is _ResultState.err:
            return Option.None_()
        return Option.Some(self._right)

    def err(self) -> Option[E]:
        """Converts from ``Result[T, E]`` to :class:`Option[E]
        <result.option.Option>`.

        Converts ``self`` into ``Option[E]``, discarding the success value, if
        any.

        Basic usage:

        >>> x: Result[int, str] = Ok(2)
        >>> x.err()
        None_
        >>>
        >>> x: Result[int, str] = Err("Nothing here")
        >>> x.err()
        Some('Nothing here')
        """
        if self._state is _ResultState.ok:
            return Option.None_()
        return Option.Some(self._left)

    def map(self, op: Callable[[T], U]) -> "Result[U, E]":
        """Maps a ``Result[T, E]`` to ``Result[U, E]`` by applying a function to
        a contained :meth:`Ok` value, leaving an :meth:`Err` value untouched.

        This function can be used to compose the results of two functions.

        Printing the numbers on each line of a string multiplied by two:

        >>> def try_parse(s: str) -> Result[int, ValueError]:
        ...     try:
        ...         return Ok(int(s))
        ...     except ValueError as e:
        ...         return Err(e)
        ...
        >>> line = "1\\n2\\n3\\n4\\n5\\n"
        >>> for num in line.splitlines():
        ...     result = try_parse(num).map(lambda i: i * 2)
        ...     if result.is_ok():
        ...         print(result.unwrap())
        2
        4
        6
        8
        10
        """
        if self._state is _ResultState.err:
            return Err(self._left)
        return Ok(op(self._right))

    def map_or(self, default: U, f: Callable[[T], U]) -> U:
        """Applies a function to the contained value (if :meth:`Ok`), or returns
        the provided default (if :meth:`Err`).

        Arguments passed to ``map_or`` are eagerly evaluated; if you are passing
        the result of a function call, it is recommended to use
        :meth:`map_or_else`, which is lazily evaluated.

        >>> x: Result[str, str] = Ok("foo")
        >>> x.map_or(42, lambda v: len(v))
        3
        >>>
        >>> x: Result[str, str] = Err("bar")
        >>> x.map_or(42, lambda v: len(v))
        42
        """
        if self._state is _ResultState.err:
            return default
        return f(self._right)

    def map_or_else(self, default: Callable[[E], U], f: Callable[[T], U]) -> U:
        """Maps a ``Result[T, E]`` to ``U`` by applying a function to a
        contained :meth:`Ok` value, or a fallback function to a contained
        :meth:`Err` value.

        This function can be used to unpack a successful result while handling
        an error.

        Basic usage:

        >>> k = 21
        >>>
        >>> x: Result[str, str] = Ok("foo")
        >>> x.map_or_else(lambda e: k * 2, lambda v: len(v))
        3
        >>>
        >>> x: Result[str, str] = Err("foo")
        >>> x.map_or_else(lambda e: k * 2, lambda v: len(v))
        42
        """
        if self._state is _ResultState.err:
            return default(self._left)
        return f(self._right)

    def map_err(self, op: Callable[[E], F]) -> "Result[T, F]":
        """Maps a ``Result[T, E]`` to ``Result[T, F]`` by applying a function to
        a contained :meth:`Err` value, leaving an :meth:`Ok` value untouched.

        This function can be used to pass through a successful result while
        handling an error.

        Basic usage:

        >>> def stringify(x: int) -> str:
        ...     return f"error code: {x}"
        ...
        >>> x: Result[int, int] = Ok(2)
        >>> x.map_err(stringify)
        Ok(2)
        >>>
        >>> x: Result[int, int] = Err(13)
        >>> x.map_err(stringify)
        Err('error code: 13')
        """
        if self._state is _ResultState.ok:
            return Ok(self._right)
        return Err(op(self._left))

    def iter(self) -> Iterator[T]:
        """Returns an iterator over the possibly contained value.

        The iterator yields one value if the reusult is :meth:`Ok`, otherwise
        none.

        Basic usage:

        >>> x: Result[int, str] = Ok(7)
        >>> next(x.iter())
        7
        >>>
        >>> x: Result[int, str] = Err("nothing!")
        >>> next(x.iter())
        Traceback (most recent call last):
            ...
        StopIteration
        """
        if self._state is _ResultState.ok:
            yield self._right

    def __iter__(self) -> Iterator[T]:
        """Returns an iterator over the possibly contained value.

        The iterator yields one value if the reusult is :meth:`Ok`, otherwise
        none.

        Basic usage:

        >>> x: Result[int, str] = Ok(7)
        >>> next(iter(x))
        7
        >>>
        >>> x: Result[int, str] = Err("nothing!")
        >>> next(iter(x))
        Traceback (most recent call last):
            ...
        StopIteration
        """
        return self.iter()

    def and_(self, res: "Result[U, E]") -> "Result[U, E]":
        """Returns ``res`` if the result is :meth:`Ok`, otherwise returns the
        :meth:`Err` value of ``self``.

        Basic usage:

        >>> x: Result[int, str] = Ok(2)
        >>> y: Result[str, str] = Err("late error")
        >>> x.and_(y)
        Err('late error')
        >>>
        >>> x: Result[int, str] = Err("early error")
        >>> y: Result[str, str] = Ok("foo")
        >>> x.and_(y)
        Err('early error')
        >>>
        >>> x: Result[int, str] = Err("not a 2")
        >>> y: Result[str, str] = Err("late error")
        >>> x.and_(y)
        Err('not a 2')
        >>>
        >>> x: Result[int, str] = Ok(2)
        >>> y: Result[str, str] = Ok("different result type")
        >>> x.and_(y)
        Ok('different result type')
        """
        if self._state is _ResultState.ok:
            return res
        return Err(self._left)

    def and_then(self, op: Callable[[T], "Result[U, E]"]) -> "Result[U, E]":
        """Calls ``op`` if the result is :meth:`Ok`, otherwise returns the
        :meth:`Err` value of ``self``.

        This function can be used for control flow based on :class:`Result`
        values.

        Basic usage:

        >>> def sq(x: int) -> Result[int, int]: return Ok(x * x)
        ...
        >>> def err(x: int) -> Result[int, int]: return Err(x)
        ...
        >>> Result[int, int].Ok(2).and_then(sq).and_then(sq)
        Ok(16)
        >>> Result[int, int].Ok(2).and_then(sq).and_then(err)
        Err(4)
        >>> Result[int, int].Ok(2).and_then(err).and_then(sq)
        Err(2)
        >>> Err(3).and_then(sq).and_then(sq)
        Err(3)
        """
        if self._state is _ResultState.err:
            return Err(self._left)
        return op(self._right)

    def or_(self, res: "Result[T, F]") -> "Result[T, F]":
        """Returns ``res`` if the result is :meth:`Err`, otherwise returns the
        :meth:`Ok` result of self.

        Arguments passed to or are eagerly evaluated; if you are passing the
        result of a function call, it is recommended to use :meth:`or_else`,
        which is lazily evaluated.

        Basic usage:

        >>> x: Result[int, str] = Ok(2)
        >>> y: Result[int, str] = Err("late error")
        >>> x.or_(y)
        Ok(2)
        >>>
        >>> x: Result[int, str] = Err("early error")
        >>> y: Result[int, str] = Ok(2)
        >>> x.or_(y)
        Ok(2)
        >>>
        >>> x: Result[int, str] = Err("not a 2")
        >>> y: Result[int, str] = Err("late error")
        >>> x.or_(y)
        Err('late error')
        >>>
        >>> x: Result[int, str] = Ok(2)
        >>> y: Result[int, str] = Ok(100)
        >>> x.or_(y)
        Ok(2)
        """
        if self._state is _ResultState.ok:
            return Ok(self._right)
        return res

    def or_else(self, op: Callable[[E], "Result[T, F]"]) -> "Result[T, F]":
        """Calls ``op`` if the result is :meth:`Err`, otherwise returns returns
        the :meth:`Ok` value from self.

        This function can be used for control flow based on result values.

        Basic usage:

        >>> def sq(x: int) -> Result[int, int]: return Ok(x * x)
        ...
        >>> def err(x: int) -> Result[int, int]: return Err(x)
        ...
        >>> Result[int, int].Ok(2).or_else(sq).or_else(sq)
        Ok(2)
        >>> Result[int, int].Ok(2).or_else(err).or_else(sq)
        Ok(2)
        >>> Result[int, int].Err(3).or_else(sq).or_else(err)
        Ok(9)
        >>> Result[int, int].Err(3).or_else(err).or_else(err)
        Err(3)
        """
        if self._state is _ResultState.ok:
            return Ok(self._right)
        return op(self._left)

    def unwrap_or(self, default: T) -> T:
        """Returns the contained :meth:`Ok` value or a provided default.

        Arguments passed to unwrap_or are eagerly evaluated; if you are passing
        the result of a function call, it is recommended to use
        :meth:`unwrap_or_else`, which is lazily evaluated.

        Basic usage:

        >>> default = 2
        >>> x: Result[int, str] = Ok(9)
        >>> x.unwrap_or(default)
        9
        >>>
        >>> x: Result[int, str] = Err("error")
        >>> x.unwrap_or(default)
        2
        """
        if self._state is _ResultState.err:
            return default
        return self._right

    def unwrap_or_else(self, op: Callable[[E], T]) -> T:
        """Returns the contained :meth:`Ok` value or computes it from a closure.

        Basic usage:

        >>> def count(x: str) -> int: return len(x)
        ...
        >>> Ok(2).unwrap_or_else(count)
        2
        >>> Result[int, str].Err("foo").unwrap_or_else(count)
        3
        """
        if self._state is _ResultState.err:
            return op(self._left)
        return self._right

    def expect(self, msg: str) -> T:
        """Returns the contained :meth:`Ok` value.

        :raises AssertionError: Raised if the value is an :meth:`Err`, with a
            message including the passed message and the content of the ``Err``.

        Basic usage:

        >>> x: Result[int, str] = Err("emergency failure")
        >>> x.expect("Testing expect")
        Traceback (most recent call last):
            ...
        AssertionError: Testing expect: 'emergency failure'
        """
        if self._state is _ResultState.err:
            raise AssertionError(f"{msg}: {repr(self._left)}")
        return self._right

    def unwrap(self) -> T:
        """Returns the contained :meth:`Ok` value, consuming the ``self`` value.

        Because this function may panic, its use is generally discouraged.
        Instead, prefer to use pattern matching and handle the Err case
        explicitly, or call :meth:`unwrap_or` or :meth:`unwrap_or_else`.

        :raises AssertionError: Raised if the value is an :meth:`Err`, with a
            message provided by the ``Err``\\'s value.

        Basic usage:

        >>> x: Result[int, str] = Ok(2)
        >>> x.unwrap()
        2
        >>>
        >>> x: Result[int, str] = Err("emergency failure")
        >>> x.unwrap()
        Traceback (most recent call last):
            ...
        AssertionError: emergency failure
        """
        if self._state is _ResultState.err:
            raise AssertionError(self._left)
        return self._right

    def expect_err(self, msg: str) -> E:
        """Returns the contained :meth:`Err` value.

        :raises AssertionError: Raised if the value is an :meth:`Ok`, with a
            message including the passed message and the content of the ``Ok``.

        Basic usage:

        >>> x: Result[int, str] = Ok(10)
        >>> x.expect_err("Testing expect_err")
        Traceback (most recent call last):
            ...
        AssertionError: Testing expect_err: 10
        """
        if self._state is _ResultState.ok:
            raise AssertionError(f"{msg}: {repr(self._right)}")
        return self._left

    def unwrap_err(self) -> E:
        """Returns the contained :meth:`Err` value.

        :raises AssertionError: Raised if the value is an :meth:`Ok`, with a
            custom message provided by the ``Ok``\\'s value.

        >>> x: Result[int, str] = Ok(2)
        >>> x.unwrap_err()
        Traceback (most recent call last):
            ...
        AssertionError: 2
        >>>
        >>> x: Result[int, str] = Err("emergency failure")
        >>> x.unwrap_err()
        'emergency failure'
        """
        if self._state is _ResultState.ok:
            raise AssertionError(self._right)
        return self._left

    __hash__ = dependent_hash(_result_value_attr)

    def __repr__(self):
        if self._state is _ResultState.ok:
            return f"Ok({repr(self._right)})"
        return f"Err({repr(self._left)})"

    @classmethod
    def Ok(cls, value: T) -> "Result[T, E]":
        """Contains the success value"""
        return cls(right=value)

    @classmethod
    def Err(cls, error: E) -> "Result[T, E]":
        """Contains the error value"""
        return cls(left=error)


Ok = Result.Ok
Err = Result.Err
